/**
 * Copyright (c) 2008-2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.engine.clock;

import java.net.URI;

import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;

import org.apache.commons.lang.CharEncoding;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.SourceUtil;

public class Listener extends AbstractJBIListener {

    @Override
    public boolean onJBIMessage(final Exchange exchange) {
        final ClockService service = ((Clock) this.getComponent()).getService();

        // Get Operation name
        final String opName = exchange.getOperation().getLocalPart();

        // Get Message Exchange Pattern
        final URI pattern = exchange.getPattern();

        if ("time".equalsIgnoreCase(opName)) {
            try {
                if (pattern.equals(MEPConstants.IN_OUT_PATTERN.value())
                        || pattern.equals(MEPConstants.IN_OPTIONAL_OUT_PATTERN.value())) {
                    final String en = service.time();
                    exchange.setOutMessageContent(SourceUtil.createSource(en));
                } else {
                    this.setIOFaultOnExchange(exchange, "Operation not allowed (" + "time"
                            + ") with the following pattern (" + exchange.getPattern().toString()
                            + ")");
                }
            } catch (JBIException e) {
                exchange.setError(e);
            }
        } else if ("printTime".equalsIgnoreCase(opName)) {
            if (!pattern.equals(MEPConstants.IN_OUT_PATTERN.value())) {
                service.printTime();
            } else {
                exchange
                        .setError(new Exception("Operation not allowed (" + "printTime"
                                + ") with the following pattern ("
                                + exchange.getPattern().toString() + ")"));
            }
        } else {
            exchange.setError(new MessagingException("Operation not allowed (" + opName
                    + ") with the following pattern (" + exchange.getPattern().toString() + ")"));
        }

        return true;
    }

    private void setIOFaultOnExchange(final Exchange exchange, final String ioMessage)
            throws JBIException {
        final StringBuffer faultMessage = new StringBuffer();
        faultMessage
                .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><tns:ioFault xmlns:tns='org.ow2.petals'><tns:message>");
        faultMessage.append(ioMessage);
        faultMessage.append("</tns:message></tns:ioFault>");
        final Fault fault = exchange.createFault();
        try {
            fault.setContent(SourceUtil.createSource(faultMessage.toString(),
                    CharEncoding.UTF_8));
            exchange.setFault(fault);
        } catch (final PEtALSCDKException e) {
            throw new JBIException(e.getMessage());
        }
    }
}
